@@ -1,18 +1,8 @@
-/* List Available Github extensions */
 #include <Rinternals.h>
-#include <R_ext/Rdynload.h>
-#include <R_ext/Visibility.h>
-#include "extensions/core-extensions.h"
 #include "syntax_extension.h"
 #include "registry.h"
 
-attribute_visible void R_init_commonmark(DllInfo *info) {
-  core_extensions_ensure_registered();
-  R_registerRoutines(info, NULL, NULL, NULL, NULL);
-  R_useDynamicSymbols(info, TRUE);
-}
-
-attribute_visible SEXP R_list_extensions(){
+SEXP R_list_extensions(){
   cmark_mem *mem = cmark_get_default_mem_allocator();
   cmark_llist *syntax_extensions = cmark_list_syntax_extensions(mem);
   cmark_llist *tmp = syntax_extensions;
